import React, { PropTypes } from 'react';
import TargetItem from './TargetItem.js';
import _find from 'lodash/find';
import _findIndex from 'lodash/findIndex';
import { DropTarget } from 'react-dnd';
import uuid from 'uuid';

import style from './style.css';


var target = {
    drop(props, monitor, component) {
        let item = monitor.getItem();
        props.onDrop(item.type, item.id, component.state.placeholderPosition);
    },
};


function collect(connect, monitor) {
    return {
        connectDropTarget: connect.dropTarget(),
        isOver:            monitor.isOver(),
    };
}


var ElementTarget = React.createClass({
    propTypes: {
        connectDropTarget: PropTypes.func.isRequired,
        isOver:            PropTypes.bool.isRequired,
        constant:          PropTypes.string.isRequired,
        elements:          PropTypes.array.isRequired,
        elementFields:     PropTypes.object.isRequired,
        elementTypes:      PropTypes.object.isRequired,
        deleteElement:     PropTypes.func.isRequired,
        renderElement:     PropTypes.func.isRequired,
        dragHint:          PropTypes.string,
        blockType:         PropTypes.string,
        onDrop:            PropTypes.func.isRequired,
        addElement:        PropTypes.func,
        moveOnTab:         PropTypes.bool,
    },
    getInitialState() {
        return {
            placeholderPosition: 0,
        };
    },
    _getPlaceholder() {
        return <div key="placeholder" className={style.placeholder} />;
    },
    _getElementField(type, id) {
        return _find(this.props.elementFields[type], item => item.id.value === id);
    },
    _toggleAllElements(show) {
        this.props.elements.forEach((element) => {
            element.isExpanded.onChange(show);
        });
    },
    _goToNextElement(currentElement) {
        if (!this.props.moveOnTab) {
            return;
        }

        const index = _findIndex(this.props.elements, { id: currentElement.id });
        if (this.props.elements[index + 1]) {
            // By changing the isExpanded we reload the whole block
            // and focus on the first element there
            this.props.elements[index + 1].isExpanded.onChange(uuid());
        } else {
            this.props.addElement(currentElement.type.value);
        }
    },
    _getElements() {
        var elements = [];
        this.props.elements.forEach((element, index) => {
            let id = element.id.value;

            if (this.props.isOver && id === this.state.placeholderPosition) {
                elements.push(this._getPlaceholder());
            }
            elements.push(
                <TargetItem
                    key={id}
                    constant={this.props.constant}
                    index={index}
                    element={element}
                    elementTypes={this.props.elementTypes}
                    elementField={this._getElementField(element.type.value, id)}
                    deleteElement={this.props.deleteElement}
                    renderElement={this.props.renderElement}
                    blockType={this.props.blockType}
                    canDrag={this.props.elements.length > 1}
                    movePlaceholderBefore={this._movePlaceholderBefore}
                    movePlaceholderAfter={this._movePlaceholderAfter}
                    toggleAllElements={this._toggleAllElements}
                    goToNextElement={this._goToNextElement}
                />
            );
        });

        if (this.props.isOver && this.state.placeholderPosition === 0) {
            elements.push(this._getPlaceholder());
        }

        if (elements.length === 0) {
            elements = <div className={style.dragDropHint}>{this.props.dragHint}</div>;
        }

        return elements;
    },
    _movePlaceholderBefore(id) {
        if (this.state.placeholderPosition === id) {
            return;
        }

        this.setState({
            placeholderPosition: id,
        });
    },
    _movePlaceholderAfter(id) {
        const elements = this.props.elements;

        const index = _findIndex(elements, element => element.id.value === id);
        const next = elements[index + 1];
        const placeholderPosition = next ? next.id.value : 0;

        if (this.state.placeholderPosition === placeholderPosition) {
            return;
        }

        this.setState({
            placeholderPosition,
        });
    },
    render() {
        var { connectDropTarget } = this.props;

        return connectDropTarget(
            <div>
                {this._getElements()}
            </div>
        );
    },
});

export default DropTarget(props => props.constant, target, collect)(ElementTarget);
