import React, { PropTypes } from 'react';
import ReactDOM from 'react-dom';
import { DropTarget, DragSource } from 'react-dnd';
import FormBlock from '../FormBlock';
import checkForErrors from '../../helpers/checkForErrors.js';
import shortenText from '../../helpers/shortenText.js';


var source = {
    beginDrag(props, monitor, component) {
        return {
            id: props.element.id.value,
        };
    },
    canDrag(props, monitor) {
        return props.canDrag;
    },
};


var target = {
    hover(props, monitor, component) {
        // Determine rectangle on screen
        var hoverBoundingRect = ReactDOM.findDOMNode(component).getBoundingClientRect();

        // Get horizontal middle
        var hoverMiddleY = (hoverBoundingRect.bottom - hoverBoundingRect.top) / 2;

        // Determine mouse position
        var clientOffset = monitor.getClientOffset();

        // Get pixels to the top
        var hoverClientY = clientOffset.y - hoverBoundingRect.top;

        if (hoverClientY < hoverMiddleY) {
            props.movePlaceholderBefore(props.element.id.value);
        } else {
            props.movePlaceholderAfter(props.element.id.value, monitor.getItem().id);
        }
    },
};


function collectTarget(connect, monitor) {
    return {
        connectDropTarget: connect.dropTarget(),
    };
}


function collectSource(connect, monitor) {
    return {
        connectDragSource: connect.dragSource(),
        isDragging:        monitor.isDragging(),
    };
}


var ElementTargetItem = React.createClass({
    propTypes: {
        connectDropTarget:     PropTypes.func.isRequired,
        connectDragSource:     PropTypes.func.isRequired,
        isDragging:            PropTypes.bool.isRequired,
        constant:              PropTypes.string,
        index:                 PropTypes.number,
        element:               PropTypes.object,
        elementTypes:          PropTypes.object,
        elementField:          PropTypes.object,
        deleteElement:         PropTypes.func,
        renderElement:         PropTypes.func,
        blockType:             PropTypes.string,
        canDrag:               PropTypes.bool,
        movePlaceholderBefore: PropTypes.func,
        movePlaceholderAfter:  PropTypes.func,
        toggleAllElements:     PropTypes.func,
        goToNextElement:       PropTypes.func,
    },
    _renderElementField() {
        let { element, elementTypes, elementField } = this.props;
        let Component = elementTypes[element.type.value].component;

        return <Component fields={elementField} />;
    },
    _deleteElement(event) {
        if (event) {
            event.preventDefault();
            event.stopPropagation();
        }

        this.props.deleteElement(this.props.index);
    },
    _renderElementActions() {
        return (
            <a href="#" onClick={this._deleteElement} title="Remove element">
                <i className="glyphicon glyphicon-remove" />
            </a>
        );
    },
    _renderElement() {
        return this.props.renderElement(this.props.element, this._renderElementField());
    },
    _onKeyPressed(event) {
        if (event.key === 'Tab') {
            event.preventDefault();
            this.props.goToNextElement(this.props.element);
        }
    },
    render() {
        var {
            connectDropTarget,
            connectDragSource,
            element,
            elementField,
            elementTypes,
            isDragging,
            blockType,
        } = this.props;

        // Don't show element if we're dragging it
        if (isDragging) {
            return null;
        }
        return connectDropTarget(
            <div onKeyDown={this._onKeyPressed}>
                <FormBlock
                    key={element.isExpanded.value.toString()}
                    title={shortenText(element.label.value)}
                    type={blockType}
                    icon={elementTypes[element.type.value].icon}
                    rightBlock={this._renderElementActions()}
                    show={element.isExpanded.value}
                    handleClick={(show, event) => {
                        if (event.shiftKey) {
                            this.props.toggleAllElements(show);
                        } else {
                            element.isExpanded.onChange(show);
                        }
                    }}
                    connectDragSource={connectDragSource}
                    isError={checkForErrors([element, elementField])}
                >
                    {this._renderElement()}
                </FormBlock>
            </div>
        );
    },
});

export default DropTarget(props => props.constant, target, collectTarget)(
    DragSource(props => props.constant, source, collectSource)(ElementTargetItem)
);
