import React, { PropTypes } from 'react';
import _max from 'lodash/max';
import _findIndex from 'lodash/findIndex';
import _forEach from 'lodash/forEach';
import SourceItem from './SourceItem.js';
import Target from './Target.js';

import style from '../../styles/common.css';


export default React.createClass({
    propTypes: {
        constant: PropTypes.string.isRequired,
        elements: PropTypes.array.isRequired,
        elementTypes: PropTypes.object.isRequired,
        elementFields: PropTypes.object.isRequired,
        renderElement: PropTypes.func.isRequired,
        dragHint: PropTypes.string,
        blockType: PropTypes.string,
        initialValues: PropTypes.object,
        moveOnTab: PropTypes.bool,
    },
    getDefaultProps() {
        return {
            dragHint: 'Drag here',
            blockType: 'info',
            initialValues: {},
            moveOnTab: false,
        };
    },
    _getNewElementId() {
        return (_max(this.props.elements.map(element => element.id.value)) || 0) + 1;
    },
    _addElement(type, index) {
        let id = this._getNewElementId();
        let element = this.props.elementTypes[type];

        this.props.elementFields[type].addField({ ...element.initialValues, id });
        this.props.elements.addField({
            id,
            type,
            label: element.defaultLabel || element.name,
            isExpanded: true,
            ...this.props.initialValues,
        }, index);
    },
    _deleteElement(index) {
        const elements = this.props.elements;

        let id = elements[index].id.value;
        let type = elements[index].type.value;

        let fieldIndex = _findIndex(this.props.elementFields[type], item => item.id.value === id);

        elements.removeField(index);
        this.props.elementFields[type].removeField(fieldIndex);
    },
    _getAddElementsButtons() {
        let buttons = [];
        _forEach(this.props.elementTypes, (item, type) => {
            buttons.push(
                <SourceItem
                    key={type}
                    constant={this.props.constant}
                    type={type}
                    name={item.name}
                    icon={item.icon}
                    addElement={this._addElement}
                />
            );
        });

        return (
            <div>{buttons}</div>
        );
    },
    _handleDrop(type, id, idBefore) {
        const { elements } = this.props;

        let indexTo = elements.length;
        if (idBefore !== 0) {
            indexTo = _findIndex(elements, element => element.id.value === idBefore);
        }

        if (id === 0) {
            this._addElement(type, indexTo);
        } else {
            const indexFrom = _findIndex(elements, element => element.id.value === id);
            if (indexFrom < indexTo) {
                indexTo--;
            }

            this._moveInArray(elements, indexFrom, indexTo);
        }
    },
    _moveInArray(array, indexFrom, indexTo) {
        if (indexFrom === indexTo) {
            return;
        }

        if (indexFrom < indexTo) {
            for (let i = indexFrom; i < indexTo; i++) {
                array.swapFields(i, i + 1);
            }
        } else {
            for (let i = indexFrom; i > indexTo; i--) {
                array.swapFields(i, i - 1);
            }
        }
    },
    render() {
        const {
            constant,
            elements,
            elementFields,
            elementTypes,
            renderElement,
            dragHint,
            blockType,
            moveOnTab,
        } = this.props;

        return (
            <div>
                <Target
                    constant={constant}
                    elements={elements}
                    elementFields={elementFields}
                    elementTypes={elementTypes}
                    onDrop={this._handleDrop}
                    deleteElement={this._deleteElement}
                    renderElement={renderElement}
                    dragHint={dragHint}
                    blockType={blockType}
                    addElement={this._addElement}
                    moveOnTab={moveOnTab}
                />
                <div className={style.buttons}>
                    {this._getAddElementsButtons()}
                </div>
            </div>
        );
    },
});
