import React from 'react';
import classnames from 'classnames';
import $ from 'jquery';

import style from './style.css';


export default React.createClass({
    propTypes: {
        title: React.PropTypes.node,
        type: React.PropTypes.string,
        comment: React.PropTypes.node,
        icon: React.PropTypes.node,
        show: React.PropTypes.oneOfType([React.PropTypes.bool, React.PropTypes.string]),
        handleClick: React.PropTypes.func,
        rightBlock: React.PropTypes.node,
        connectDragSource: React.PropTypes.func,
        children: React.PropTypes.node,
        isError: React.PropTypes.bool,
    },
    getDefaultProps() {
        return {
            // Title to show on the link
            title: 'Hidden block',

            // Type of the block
            // Options: 'default', 'primary', 'success', 'info', ' warning', 'danger'
            type: 'default',

            // Comment to show after the link
            comment: null,

            // Class for icon
            icon: null,

            // Show the hidden block or not?
            // If handleClick is null then it will be the default value.
            // If handleClick is a function then it will be the actual value.
            show: false,

            // Callback when clicking on the link
            handleClick: null,

            // Something to show on the right side
            rightBlock: null,

            // Is there some error inside the block
            isError: false,
        };
    },
    getInitialState() {
        return {
            show: this.props.show,
        };
    },
    componentDidUpdate(prevProps, prevState) {
        if (this.props.handleClick) {
            if (prevProps.show !== this.props.show && this.props.show) {
                this._focusOnFirstField();
            }
        } else if (prevState.show !== this.state.show && this.state.show) {
            this._focusOnFirstField();
        }
    },
    _focusOnFirstField() {
        $(this.refs.body).find('input[type="text"],textarea').first().focus().select();
    },
    _clicked(event) {
        event.preventDefault();

        if (this.props.handleClick) {
            this.props.handleClick(!this.props.show, event);
        } else {
            this.setState({
                show: !this.state.show,
            });
        }
    },
    _isShow() {
        if (this.props.handleClick) {
            return this.props.show;
        }

        return this.state.show;
    },
    _renderHeader() {
        let arrowClassName = this._isShow() ? 'glyphicon glyphicon-triangle-bottom' : 'glyphicon glyphicon-triangle-right';

        let headingClassName = classnames([
            'panel-heading',
            style.heading,
            { [style.headingAlone]: !this._isShow() },
        ]);

        let header = (
            <div className={headingClassName} onClick={this._clicked}>
                <span className={style.arrowWrapper}>
                    <span className={arrowClassName} />
                </span>

                {this.props.icon && (
                    <div className={style.icon}>{this.props.icon}</div>
                )}

                <span className={style.link}>{this.props.title}</span>
                {this.props.comment}
                {(this.props.rightBlock || this.props.isError) &&
                    <div className={style.rightBlock}>
                        {this.props.isError &&
                            <i className={'glyphicon glyphicon-warning-sign ' + style.error} />
                        }
                        {this.props.rightBlock}
                    </div>
                }
            </div>
        );
        if (this.props.connectDragSource) {
            header = this.props.connectDragSource(header);
        }

        return header;
    },
    render() {
        let panelClassName = classnames([
            'panel',
            'panel-' + this.props.type,
            style.panel,
        ]);

        return (
            <div className={panelClassName}>
                {this._renderHeader()}
                {this._isShow() &&
                    <div className="panel-body" ref="body">
                        {this.props.children}
                    </div>
                }
            </div>
        );
    },
});
