import React, { PropTypes } from 'react';
import FormBlock from '../../../components/FormBlock';
import _max from 'lodash/max';
import _find from 'lodash/find';
import _keys from 'lodash/keys';
import Operator from './Operator';
import Value from './Value';
import style from './style.css';
import operators from './operators.js';
import commonStyle from '../../../styles/common.css';


export const validate = (values) => {
    const errors = {};

    values.showConditionsRules.forEach((rule, index) => {
        if (!rule.dataConnector) {
            errors.showConditionsRules = errors.showConditionsRules || [];
            errors.showConditionsRules[index] = errors.showConditionsRules[index] || {};
            errors.showConditionsRules[index].dataConnector = 'Choose data connector';
        } else if (!rule.operator || rule.operator === '0') {
            errors.showConditionsRules = errors.showConditionsRules || [];
            errors.showConditionsRules[index] = errors.showConditionsRules[index] || {};
            errors.showConditionsRules[index].operator = 'Choose operator';
        }
    });

    return errors;
};


export default React.createClass({
    propTypes: {
        fields: PropTypes.object.isRequired,
        formBlockType: PropTypes.string.isRequired,
        DataConnectorsPicker: PropTypes.func.isRequired,
    },
    getNewRuleId() {
        return (_max(this.props.fields.showConditionsRules.map(rule => rule.id.value)) || 0) + 1;
    },
    addRule(event) {
        if (event) {
            event.preventDefault();
        }

        this.props.fields.showConditionsRules.addField({
            id: this.getNewRuleId(),
            dataConnector: null,
            operator: null,
            value: '',
        });
    },
    deleteRule(index) {
        this.props.fields.showConditionsRules.removeField(index);
    },
    afterDataConnectorChange(index) {
        const { operator, value } = this.props.fields.showConditionsRules[index];

        // Nullify operator and value
        operator.onChange(null);
        value.onChange(null);
    },
    getOperatorDefinition(rule) {
        if (!rule.dataConnector.value || !rule.operator.value) {
            return null;
        }

        return _find(operators[rule.dataConnector.value.type], { code: rule.operator.value });
    },
    getOperatorsTypes() {
        return _keys(operators);
    },
    render() {
        const {
            showConditionsExpanded,
            showConditionsRules,
        } = this.props.fields;
        const { DataConnectorsPicker } = this.props;

        return (
            <FormBlock
                title="Show conditions"
                type={this.props.formBlockType}
                show={showConditionsExpanded.value}
                handleClick={show => showConditionsExpanded.onChange(show)}
            >
                {showConditionsRules.map((rule, index) => (
                    <div key={rule.id.value} className={'alert alert-info ' + style.ruleWrapper}>
                        <a
                            className={style.delete}
                            onClick={this.deleteRule.bind(this, index)}
                            title="Delete condition"
                        >
                            &times;
                        </a>
                        <div className={!rule.dataConnector.value && commonStyle.lastRow}>
                            <DataConnectorsPicker
                                field={rule.dataConnector}
                                label=""
                                type={this.getOperatorsTypes()}
                                afterChange={this.afterDataConnectorChange.bind(this, index)}
                            />
                        </div>
                        <div className={style.rule}>
                            <Operator
                                dataConnector={rule.dataConnector.value}
                                field={rule.operator}
                            />
                            <Value
                                operator={this.getOperatorDefinition(rule)}
                                field={rule.value}
                            />
                        </div>
                    </div>
                ))}
                <a className="btn btn-default btn-sm" onClick={this.addRule}>Add condition</a>
            </FormBlock>
        );
    },
});
