/* global describe, it, expect */
import operators, { evaluateCondition } from './operators.js';
import _find from 'lodash/find';


describe('Array operators', () => {
    it('Should evaluate "includes"', () => {
        const evaluate = _find(operators.array, { code: 'includes' }).evaluate;
        expect(evaluate([1, 2], 1)).toBe(true);
        expect(evaluate([1, 2], 3)).toBe(false);
        expect(evaluate(undefined, 3)).toBe(false);
    });

    it('Should evaluate "doesntInclude"', () => {
        const evaluate = _find(operators.array, { code: 'doesntInclude' }).evaluate;
        expect(evaluate([1, 2], 1)).toBe(false);
        expect(evaluate([1, 2], 3)).toBe(true);
        expect(evaluate(undefined, 3)).toBe(true);
    });
});


describe('Number operators', () => {
    it('Should evaluate "<"', () => {
        const evaluate = _find(operators.number, { code: '<' }).evaluate;
        expect(evaluate(null, 5)).toBe(false);
        expect(evaluate('456', 5)).toBe(false);
        expect(evaluate(4, 5)).toBe(true);
        expect(evaluate(5, 5)).toBe(false);
        expect(evaluate(6, 5)).toBe(false);
    });

    it('Should evaluate "<="', () => {
        const evaluate = _find(operators.number, { code: '<=' }).evaluate;
        expect(evaluate(null, 5)).toBe(false);
        expect(evaluate('456', 5)).toBe(false);
        expect(evaluate(4, 5)).toBe(true);
        expect(evaluate(5, 5)).toBe(true);
        expect(evaluate(6, 5)).toBe(false);
    });

    it('Should evaluate "="', () => {
        const evaluate = _find(operators.number, { code: '=' }).evaluate;
        expect(evaluate(null, 5)).toBe(false);
        expect(evaluate('456', 5)).toBe(false);
        expect(evaluate(4, 5)).toBe(false);
        expect(evaluate(5, 5)).toBe(true);
        expect(evaluate(6, 5)).toBe(false);
    });

    it('Should evaluate ">="', () => {
        const evaluate = _find(operators.number, { code: '>=' }).evaluate;
        expect(evaluate(null, 5)).toBe(false);
        expect(evaluate('456', 5)).toBe(false);
        expect(evaluate(4, 5)).toBe(false);
        expect(evaluate(5, 5)).toBe(true);
        expect(evaluate(6, 5)).toBe(true);
    });

    it('Should evaluate ">"', () => {
        const evaluate = _find(operators.number, { code: '>' }).evaluate;
        expect(evaluate(null, 5)).toBe(false);
        expect(evaluate('456', 5)).toBe(false);
        expect(evaluate(4, 5)).toBe(false);
        expect(evaluate(5, 5)).toBe(false);
        expect(evaluate(6, 5)).toBe(true);
    });
});


describe('String operators', () => {
    it('Should evaluate "defined"', () => {
        const evaluate = _find(operators.string, { code: 'defined' }).evaluate;
        expect(evaluate('string')).toBe(true);
        expect(evaluate('')).toBe(false);
        expect(evaluate(undefined)).toBe(false);
    });

    it('Should evaluate "undefined"', () => {
        const evaluate = _find(operators.string, { code: 'undefined' }).evaluate;
        expect(evaluate('string')).toBe(false);
        expect(evaluate('')).toBe(true);
        expect(evaluate(undefined)).toBe(true);
    });

    it('Should evaluate "includes"', () => {
        const evaluate = _find(operators.string, { code: 'includes' }).evaluate;
        expect(evaluate('string', 'tri')).toBe(true);
        expect(evaluate('string', 'pre')).toBe(false);
        expect(evaluate(undefined, 'tri')).toBe(false);
    });

    it('Should evaluate "doesntIinclude"', () => {
        const evaluate = _find(operators.string, { code: 'doesntInclude' }).evaluate;
        expect(evaluate('string', 'tri')).toBe(false);
        expect(evaluate('string', 'pre')).toBe(true);
        expect(evaluate(undefined, 'tri')).toBe(true);
    });

    it('Should evaluate "startsWith"', () => {
        const evaluate = _find(operators.string, { code: 'startsWith' }).evaluate;
        expect(evaluate('string', 'str')).toBe(true);
        expect(evaluate('string', 'STr')).toBe(true);
        expect(evaluate('string', 'ttt')).toBe(false);
        expect(evaluate('string', 'tri')).toBe(false);
        expect(evaluate(undefined, 'str')).toBe(false);
    });

    it('Should evaluate "endsWith"', () => {
        const evaluate = _find(operators.string, { code: 'endsWith' }).evaluate;
        expect(evaluate('string', 'ing')).toBe(true);
        expect(evaluate('string', 'iNG')).toBe(true);
        expect(evaluate('string', 'ttt')).toBe(false);
        expect(evaluate('string', 'tri')).toBe(false);
        expect(evaluate(undefined, 'str')).toBe(false);
    });

    it('Should evaluate "regex"', () => {
        const evaluate = _find(operators.string, { code: 'regex' }).evaluate;
        expect(evaluate('string', 'ing')).toBe(true);
        expect(evaluate('string', 'iNG')).toBe(true);
        expect(evaluate('string', '^st')).toBe(true);
        expect(evaluate('string', '.')).toBe(true);
        expect(evaluate('string', 'ttt')).toBe(false);
    });
});


describe('evaluateCondition', () => {
    it('Should return evaluate result', () => {
        expect(evaluateCondition({
            dataConnector: [1, 2],
            type: 'array',
            operator: 'includes',
            value: 1,
        })).toBe(true);
    });
});
