import React, { PropTypes } from 'react';
import { reduxForm } from 'redux-form';
import _isEqual from 'lodash/isEqual';
import _isFunction from 'lodash/isFunction';
import _omit from 'lodash/omit';
import { shouldUpdate } from 'recompose';


export default ({ validate = () => {}, fields = ['value'] }) => {
    return (WrappedComponent) => {
        const ReduxForm = reduxForm({})(React.createClass({
            propTypes: {
                originalProps: PropTypes.object,
                fields: PropTypes.object,
                isValidating: PropTypes.oneOfType([
                    PropTypes.bool,
                    PropTypes.number,
                ]),
                touchAll: PropTypes.func.isRequired,
                onUpdate: PropTypes.func,
                onError: PropTypes.func,
                values: PropTypes.object,
                valid: PropTypes.bool,
                errors: PropTypes.object,
            },
            getDefaultProps() {
                return {
                    registerAction() {},
                };
            },
            componentDidMount() {
                const { valid, errors } = this.props;

                if (!valid) {
                    this.props.onError(errors.value);
                }
            },
            componentDidUpdate(prevProps) {
                const { valid, values, errors } = this.props;

                if (this.props.isValidating && this.props.isValidating !== prevProps.isValidating) {
                    this.props.touchAll();
                }


                // Return new values or errors
                if (_isEqual(prevProps.valid, valid) && _isEqual(prevProps.values, values)) {
                    return;
                }

                if (!valid) {
                    this.props.onError(errors.value);
                } else {
                    this.props.onUpdate(values.value);
                }
            },
            render() {
                return (
                    <WrappedComponent
                        field={this.props.fields.value}
                        {..._omit(this.props.originalProps, [
                            'initialValues',
                            'form',
                            'isValidating',
                            'onUpdate',
                            'onError',
                        ])}
                    />
                );
            },
        }));


        return shouldUpdate((props, nextProps) => {
            // Update only if it's a preview mode or settings have been changed
            return props.isPreview || !_isEqual(props.settings, nextProps.settings);
        })(React.createClass({
            propTypes: {
                form: React.PropTypes.string,
                settings: React.PropTypes.object,
                initialValues: React.PropTypes.object,
                isPreview: React.PropTypes.bool,
                onUpdate: React.PropTypes.func,
                onError: React.PropTypes.func,
            },
            getDefaultProps() {
                return {
                    form: 'playerForm',
                    settings: {},
                    initialValues: {},
                    isPreview: false,
                    onUpdate() {},
                    onError() {},
                };
            },
            _validate(values) {
                const errors = {};

                const error = validate(values.value, this.props.settings);
                if (error) {
                    errors.value = error;
                }

                return errors;
            },
            _getFields() {
                if (_isFunction(fields)) {
                    return fields(this.props.settings);
                }

                return fields;
            },
            render() {
                return (
                    <ReduxForm
                        originalProps={this.props}
                        form={this.props.form}
                        initialValues={this.props.initialValues}
                        validate={this._validate}
                        readonly={this.props.isPreview}
                        fields={this._getFields()}
                        onUpdate={this.props.onUpdate}
                        onError={this.props.onError}
                        overwriteOnInitialValuesChange={this.props.isPreview}
                    />
                );
            },
        }));
    };
};
